// CK 7. January 2011

// This widgets shuffles all unlocked objects on a page by swaping their x,y positions
// The new x,y can be set directly or the object moves to the new position in animation


var animated = true;         // flag whether position swap is animated
var blockAnimation = false;  // avoid that shuffle is re-started while animating (otherwise we loose the original positions)


function Coordinate (x,y) {
     
    this.x = x;
    this.y = y;
    
}


// called by the widget GUI to turn on/off animation
function switchAnimationFlag() {
    
    animated = !animated; // switch
    
    // update checkbox image:
    if (animated)
       document.getElementById("animatedFlag").src = "checked.png";
    else
        document.getElementById("animatedFlag").src = "unchecked.png";


}


// shuffle is the core method that calculates the new positions / swaps

function shuffle ( ) {
    
    //  do not swap while animating!
    if (blockAnimation) return;
    blockAnimation = true;
    
    
    // get all objects
    var allObjects = NB.page.getObjectsAsArray();
        
    // make an array of objects that should be shuffled (un-locked, and not the widget!)
    var shuffleObjectsSource = new Array(); // to get original positions
    var shuffleObjectsTarget = new Array(); // to get new positions
    
    for (var i = 0; i < allObjects.length; ++i) {
        
        if ( allObjects[i].type != "widget"  && allObjects[i].lockType != "Lock In Place") {
            
            console.log ("Adding: " + allObjects[i]);
	    // we need a copy of the original coordinates because each will be set to new coordinates
	    // therefore we store coordinates instead of the object reference
            shuffleObjectsSource.push(new Coordinate ( allObjects[i].x , allObjects[i].y ) ); 
            shuffleObjectsTarget.push(allObjects[i]);
                        
            
        }
        
        
    }
        
    
    // swap n times:
    console.log ("Shuffle")
    
    
    // Each object will be swaped to a new position:

    var limit = shuffleObjectsTarget.length;
    for (var i =0 ; i < shuffleObjectsTarget.length; ++i) {
        
        // new spot for i-th element:
        var random = Math.floor(Math.random() * limit);
        console.log ("Swap " + i + " with " + random);
        
        var store = shuffleObjectsTarget[i];
        shuffleObjectsTarget[i] = shuffleObjectsTarget[random];
        shuffleObjectsTarget[random] = store;
        
    }
           
    
    // Execute the new positions:
    
    if (!animated)  { NB.beginBatch(); }  // set all objects to new positions at once
    
    // Setting the new positions:
    for (var i = 0; i < shuffleObjectsTarget.length ; ++i) {
                
        if (animated) {
        
	    setButtonImage();  // indicate that objects are shuffled and no new shuffle can be started
	 
            var x = shuffleObjectsSource[i].x;
            var y = shuffleObjectsSource[i].y;
                        
            // Creates a new animation process that ends when all steps are taken:
            new flyIn (shuffleObjectsTarget[i].guid , x, y  );
        
                
        } else {
            
	   // no animation, therefore set x,y directly: 
           shuffleObjectsTarget[i].x = shuffleObjectsSource[i].x;
           shuffleObjectsTarget[i].y = shuffleObjectsSource[i].y;
	   
            
        } // end if animated
        
        
        
    }
    
    if (!animated) { NB.endBatch(); blockAnimation = false; } // if not animated, set all object positions at once
    
    
    
    
}



// update GUI
function setButtonImage() {
    
    if (blockAnimation)
       document.getElementById("shuffleButton").src = "shuffleRun.png";
    else
        document.getElementById("shuffleButton").src = "shuffleStart.png";
    
}


// Will be called when an animation stops:
function stopBlocking () {
    
    blockAnimation = false;
    setButtonImage();    
    
}


////////////////////////////////////////////////////////////////////////////////

// Animation stuff /

// Timer method (see selfhtml)
Function.prototype.Timer = function (interval, calls, onend) {
  var count = 0;
  var payloadFunction = this;
  var startTime = new Date();
  var callbackFunction = function () {
    return payloadFunction(startTime, count);
  };
  var endFunction = function () {
    if (onend) {
      onend(startTime, count, calls);
    }
  };
  var timerFunction = function () {
    count++;
    if (count < calls && callbackFunction() != false) {
      window.setTimeout(timerFunction, interval);
    } else {
      endFunction();
    }
  };
  timerFunction();
};
// end code from self html


// change this value to set up the animation speeed:
var numberOfSteps = 10;


// moves an oject step by step to the target position; will be called
// frequently by timer method
// objId the guid of the object to animate
// targetX the final xposition of the animation process
// targetY the final yposition of the animation process


function flyIn (objId , targetX, targetY) {
       
   this.objId = objId; 
    
   var startProxy = NB.getObject (objId);
   startX = startProxy.x;
   startY = startProxy.y;
      
   
   // splot total distance by number of steps
   var stepX = (targetX - startX ) / numberOfSteps;
   var stepY = (targetY - startY ) / numberOfSteps;
   
    
   var myCounter = "Start"; // just for debugging    
    
    
   // the function is called by the timer and executes the animation step: 
   var step = function() {		
	
	var proxy = NB.getObject (  objId ); // get latest proxy
        
        NB.beginBatch();
	  proxy.x = proxy.x + stepX;
          proxy.y = proxy.y + stepY;
        NB.endBatch();
        
        // debug information:
        console.log ( objId + " moves to: " + proxy.x + ","+ proxy.y )       
        myCounter = myCounter+" -  step ";
        console.log ( objId + " " + myCounter);
        
    }
    
    step.Timer ( 10 , numberOfSteps + 1, function (){ stopBlocking() }  ) ;
    
}





